//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
module tsopc;
  
  reg clk_p_i;
  reg clk_n_i;
  reg clk_i;
  reg [3:0] btn_i;
  reg [3:0] sw_i;
  wire odt_o;
  wire reset_n_o;
  wire cke_o;
  wire [12:0]  adr_o;
  wire [1:0] adr_na_o;
  wire clk_p_o;
  wire clk_n_o;
  wire ras_n_o;
  wire cas_n_o;
  wire we_n_o;
  wire [2:0] ba_o;
  wire ldm_o;
  wire udm_o;
  wire [15:0]  dq_io;
  wire ldqs_p_io;
  wire ldqs_n_io;
  wire udqs_p_io;
  wire udqs_n_io;
    // UART interface
  reg  sin_i;
  wire sout_o;
    // Test I/O
  wire [3:0] led_o; 
  
  sopc dut (
  .clk_p_i                (clk_p_i), 
  .clk_n_i                (clk_n_i), 
  .clk_i                  (clk_i), 
  .btn_i                  (btn_i), 
  .sw_i                   (sw_i),
  // DDR3 interface						
  .odt_o                  (odt_o), 
  .reset_n_o              (reset_n_o), 
  .cke_o                  (cke_o), 
  .adr_o                  (adr_o), 
  .adr_na_o               (adr_na_o), 
  .clk_p_o                (clk_p_o), 
  .clk_n_o                (clk_n_o),
  .ras_n_o                (ras_n_o), 
  .cas_n_o                (cas_n_o), 
  .we_n_o                 (we_n_o), 
  .ba_o                   (ba_o), 
  .ldm_o                  (ldm_o), 
  .udm_o                  (udm_o),
  .dq_io                  (dq_io), 
  .ldqs_p_io              (ldqs_p_io), 
  .ldqs_n_io              (ldqs_n_io), 
  .udqs_p_io              (udqs_p_io), 
  .udqs_n_io              (udqs_n_io),
  // UART interface
  .sin_i                  (sin_i), 
  .sout_o                 (sout_o),
  // Test I/O
  .led_o                  (led_o)   
  );
  
  initial
    begin
      clk_i = 1'b0;
      sin_i = 1'b1;
      clk_p_i = 1'b0;
      clk_n_i = 1'b1;
      btn_i = 4'h0;
    //-------------------------UART data transfer---------------------------
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b1;        // 8'h33      - MWRITE command
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //--Address-------
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'hF0
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //-- Size
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h01
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //-----Data-----------------
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h01
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h03
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  

    
    
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b1;        // 8'h33      - MWRITE command
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //--Address-------
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h80
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //-- Size
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h02
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //-----Data-----------------
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h81
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h02
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h05
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h03
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
  //--Second word
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h97
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h06
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h04
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h23
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //------------End of MWRITE command

    //------------MREAD command-----------------------
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;        // 8'h34      - MREAD command
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //--Address-------
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h80
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;            // Start bit
      #640 sin_i = 1'b0;       // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //-- Size
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b0;        // 8'h00
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit  
    //
      #800 sin_i = 1'b0;         // Start bit
      #640 sin_i = 1'b1;        // 8'h09
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b1;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;
      #640 sin_i = 1'b0;       
      #640 sin_i = 1'b1;       // Stop bit      

    //----------End of MREAD command------------------
    
    //-----------------------End of UART data transfer----------------------
    end
    
  always
    begin
      #10 clk_i = ~clk_i;  
    end
    
  always
    begin
      #2 clk_p_i = ~clk_p_i;
         clk_n_i = ~clk_n_i;
    end
    
endmodule