//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
module clk_mgmt (
  hclk_i, gclk_i, rst_i, stb_i, dm_i, cmd_i, adr_i, tri_i,
  clk_p_o, clk_n_o, dm_o, cmd_o, adr_o, dqs_p_o, dqs_n_o
  );
  input hclk_i;                        // Input high frequency clock
  input gclk_i;                        // Input global clock - low frequency input to SERDES
  input rst_i;                         // Input reset signal - high active
  input stb_i;                         // Input data strobe from BUFPLL
  input [7:0] dm_i;                    // Input data mask
  input [2:0] cmd_i;
  input [15:0] adr_i;
  input tri_i;                         // Tristate input
  output clk_p_o;                      // Output positive clock to DDR3
  output clk_n_o;                      // Output negative clock to DDR3
  output [1:0] dm_o;                   // Output data mask
  output [2:0] cmd_o;
  output [15:0] adr_o;
  output [1:0] dqs_p_o;
  output [1:0] dqs_n_o;
  //-------------------Internal wires and registers-----------------------------------

  wire [1:0] dm_out_w;
  wire [1:0] dmd_w;

  wire [1:0] dm1_out = dm_i[1:0];
  wire [1:0] dm2_out = dm_i[3:2];
  wire [1:0] dm3_out = dm_i[5:4];
  wire [1:0] dm4_out = dm_i[7:6];

  wire ddr_clk_fwd_w;
	 
  wire ddr_clk_out_w;
  wire tq_w;
  
  wire [2:0] cmd_w;
  wire [2:0] tri_w;
  wire [2:0] cmd_d_w;
  
  wire [15:0] adr_w;
  wire [15:0] tria_w;
  wire [15:0] adr_d_w;

  wire [1:0] dqs_pout_w;
  wire [1:0] dqs_nout_w;
  wire [1:0] dqs_tpout_w;
  wire [1:0] dqs_tnout_w;
  wire [1:0] dqs_pdly_w;
  wire [1:0] dqs_ndly_w;
  wire [1:0] dqs_tpdly_w;
  wire [1:0] dqs_tndly_w;	

  genvar i;
//--------------------------------Main clock forward logic-------------------------------  
        OSERDES2 #(
      .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
      .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
      .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
      .DATA_WIDTH(4),               // Parallel data width (2-8)
      .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
      .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
      .TRAIN_PATTERN(0)             // Training Pattern (0-15)
   )
   OSERDES2_clk_fwd (
      .OQ(ddr_clk_fwd_w),               // 1-bit Data output to pad or IODELAY2
      .SHIFTOUT1(), // 1-bit Cascade data output
      .SHIFTOUT2(), // 1-bit Cascade 3-state output
      .SHIFTOUT3(), // 1-bit Cascade differential data output
      .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
      .TQ(TQ),               // 1-bit 3-state output to pad or IODELAY2
      .CLK0(hclk_i),           // 1-bit I/O clock input
      .CLK1(),           // 1-bit Secondary I/O clock input
      .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
      // D1 - D4: 1-bit (each) Parallel data inputs
      .D1(1'b0),
      .D2(1'b1),
      .D3(1'b0),
      .D4(1'b1),
      .IOCE(stb_i),           // 1-bit Data strobe input
      .OCE(1'b1),             // 1-bit Clock enable input
      .RST(rst_i),             // 1-bit Asynchrnous reset input
      .SHIFTIN1(),   // 1-bit Cascade data input
      .SHIFTIN2(),   // 1-bit Cascade 3-state input
      .SHIFTIN3(),   // 1-bit Cascade differential data input
      .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
      // T1 - T4: 1-bit (each) 3-state control inputs
      .T1(1'b0),
      .T2(1'b0),
      .T3(1'b0),
      .T4(1'b0),
      .TCE(1'b1),             // 1-bit 3-state clock enable input
      .TRAIN(1'b0)          // 1-bit Training pattern enable input
   );   	 

     IODELAY2 #(
      .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
      .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
      .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
      .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
      .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
      .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                         // or "DIFF_PHASE_DETECTOR" 
      .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
      .ODELAY_VALUE(0),                  // Amount of taps fixed output delay (0-255)
      .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
      .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
   )
   IODELAY2_clk_fwd_dly (
      .BUSY(),         // 1-bit Busy output after CAL
      .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
      .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
      .DOUT(ddr_clk_out_w),         // 1-bit Delayed data output
      .TOUT(TOUT),         // 1-bit Delayed 3-state output
      .CAL(1'b0),           // 1-bit Initiate calibration input
      .CE(1'b0),             // 1-bit Enable INC input
      .CLK(gclk_i),           // 1-bit Clock input
      .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
      .INC(1'b0),           // 1-bit Increment / decrement input
      .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
      .IOCLK1(),     // 1-bit Input from the I/O clock network
      .ODATAIN(ddr_clk_fwd_w),   // 1-bit Output data input from output register or OSERDES2.
      .RST(rst_i),           // 1-bit Reset to zero or 1/2 of total delay period
      .T(T)                // 1-bit 3-state input signal
   );			  

				
     OBUFDS #(
      .IOSTANDARD("DIFF_SSTL15_II") // Specify the output I/O standard
   ) OBUFDS_ddr_clk (
      .O(clk_p_o),     // Diff_p output (connect directly to top-level port)
      .OB(clk_n_o),   // Diff_n output (connect directly to top-level port)
      .I(ddr_clk_out_w)      // Buffer input 
   ); 
//-------------------End of main clock forward logic---------------------------
//
//-------------------Data mask forward logic-----------------------------------
generate
  for (i=0; i<2; i=i+1)
    begin: obuf_dm
    OSERDES2 #(
    .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
    .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
    .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
    .DATA_WIDTH(4),               // Parallel data width (2-8)
    .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
    .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
    .TRAIN_PATTERN(0)             // Training Pattern (0-15)
 )
 OSERDES2_obuf_dm (
    .OQ(dm_out_w[i]),               // 1-bit Data output to pad or IODELAY2
    .SHIFTOUT1(), // 1-bit Cascade data output
    .SHIFTOUT2(), // 1-bit Cascade 3-state output
    .SHIFTOUT3(), // 1-bit Cascade differential data output
    .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
    .TQ(TQ),               // 1-bit 3-state output to pad or IODELAY2
    .CLK0(hclk_i),           // 1-bit I/O clock input
    .CLK1(),           // 1-bit Secondary I/O clock input
    .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
    // D1 - D4: 1-bit (each) Parallel data inputs
    .D1(dm1_out[i]),
    .D2(dm2_out[i]),
    .D3(dm3_out[i]),
    .D4(dm4_out[i]),
    .IOCE(stb_i),           // 1-bit Data strobe input
    .OCE(1'b1),             // 1-bit Clock enable input
    .RST(rst_i),             // 1-bit Asynchrnous reset input
    .SHIFTIN1(),   // 1-bit Cascade data input
    .SHIFTIN2(),   // 1-bit Cascade 3-state input
    .SHIFTIN3(),   // 1-bit Cascade differential data input
    .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
    // T1 - T4: 1-bit (each) 3-state control inputs
    .T1(1'b0),
    .T2(1'b0),
    .T3(1'b0),
    .T4(1'b0),
    .TCE(1'b1),             // 1-bit 3-state clock enable input
    .TRAIN(1'b0)          // 1-bit Training pattern enable input
 );   	 

IODELAY2 #(
 .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
 .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
 .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
 .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
 .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
 .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                    // or "DIFF_PHASE_DETECTOR" 
 .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
 .ODELAY_VALUE(30),                  // Amount of taps fixed output delay (0-255)
 .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
 .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
)
IODELAY2_dm_delay (
 .BUSY(),         // 1-bit Busy output after CAL
 .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
 .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
 .DOUT(dmd_w[i]),         // 1-bit Delayed data output
 .TOUT(TOUT),         // 1-bit Delayed 3-state output
 .CAL(1'b0),           // 1-bit Initiate calibration input
 .CE(1'b0),             // 1-bit Enable INC input
 .CLK(gclk_i),           // 1-bit Clock input
 .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
 .INC(1'b0),           // 1-bit Increment / decrement input
 .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
 .IOCLK1(),     // 1-bit Input from the I/O clock network
 .ODATAIN(dm_out_w[i]),   // 1-bit Output data input from output register or OSERDES2.
 .RST(rst_i),           // 1-bit Reset to zero or 1/2 of total delay period
 .T(T)                // 1-bit 3-state input signal
);			  

OBUF #(
   .DRIVE(12),   // Specify the output drive strength
   .IOSTANDARD("SSTL15_II"), // Specify the output I/O standard
   .SLEW("FAST") // Specify the output slew rate
) OBUF_dm_obuf (
   .O(dm_o[i]),     // Buffer output (connect directly to top-level port)
   .I(dmd_w[i])      // Buffer input 
);
    end
endgenerate
//-----------------End of data mask forward logic-----------------------------
//
//
//---------------------------------Cmd forward-------------------------------------------------

generate
  for (i=0; i<3; i=i+1)
    begin: cmd_obuf
    OSERDES2 #(
    .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
    .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
    .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
    .DATA_WIDTH(4),               // Parallel data width (2-8)
    .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
    .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
    .TRAIN_PATTERN(0)             // Training Pattern (0-15)
 )
 OSERDES2_cmd_obuf (
    .OQ(cmd_w[i]),               // 1-bit Data output to pad or IODELAY2
    .SHIFTOUT1(), // 1-bit Cascade data output
    .SHIFTOUT2(), // 1-bit Cascade 3-state output
    .SHIFTOUT3(), // 1-bit Cascade differential data output
    .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
    .TQ(tri_w[i]),               // 1-bit 3-state output to pad or IODELAY2
    .CLK0(hclk_i),           // 1-bit I/O clock input
    .CLK1(),           // 1-bit Secondary I/O clock input
    .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
    // D1 - D4: 1-bit (each) Parallel data inputs
    .D1(cmd_i[i]),
    .D2(cmd_i[i]),
    .D3(1'b1),
    .D4(1'b1),
    .IOCE(stb_i),           // 1-bit Data strobe input
    .OCE(1'b1),             // 1-bit Clock enable input
    .RST(rst_i),             // 1-bit Asynchrnous reset input
    .SHIFTIN1(),   // 1-bit Cascade data input
    .SHIFTIN2(),   // 1-bit Cascade 3-state input
    .SHIFTIN3(),   // 1-bit Cascade differential data input
    .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
    // T1 - T4: 1-bit (each) 3-state control inputs
    .T1(1'b0),
    .T2(1'b0),
    .T3(1'b0),
    .T4(1'b0),
    .TCE(1'b1),             // 1-bit 3-state clock enable input
    .TRAIN(1'b0)          // 1-bit Training pattern enable input
 ); 
 
 IODELAY2 #(
  .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
  .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
  .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
  .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
  .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
  .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                     // or "DIFF_PHASE_DETECTOR" 
  .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
  .ODELAY_VALUE(0),                  // Amount of taps fixed output delay (0-255)
  .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
  .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
)
IODELAY2_cmd_dly (
  .BUSY(),         // 1-bit Busy output after CAL
  .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
  .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
  .DOUT(cmd_d_w[i]),         // 1-bit Delayed data output
  .TOUT(TOUT),         // 1-bit Delayed 3-state output
  .CAL(1'b0),           // 1-bit Initiate calibration input
  .CE(1'b0),             // 1-bit Enable INC input
  .CLK(gclk_i),           // 1-bit Clock input
  .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
  .INC(1'b0),           // 1-bit Increment / decrement input
  .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
  .IOCLK1(),     // 1-bit Input from the I/O clock network
  .ODATAIN(cmd_w[i]),   // 1-bit Output data input from output register or OSERDES2.
  .RST(rst_i),           // 1-bit Reset to zero or 1/2 of total delay period
  .T(tri_w[i])                // 1-bit 3-state input signal
);			  

OBUF #(
   .DRIVE(12),   // Specify the output drive strength
   .IOSTANDARD("SSTL15_II"), // Specify the output I/O standard
   .SLEW("FAST") // Specify the output slew rate
) OBUF_cmd_out (
   .O(cmd_o[i]),     // Buffer output (connect directly to top-level port)
   .I(cmd_d_w[i])      // Buffer input 
);

    end
endgenerate  

//--------------------------------End of cmd forward-------------------------------------------
//
//------------------------------Address forward------------------------------------------------
generate
  for (i=0; i<15; i=i+1)
    begin: adr_obuf
    OSERDES2 #(
    .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
    .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
    .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
    .DATA_WIDTH(4),               // Parallel data width (2-8)
    .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
    .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
    .TRAIN_PATTERN(0)             // Training Pattern (0-15)
 )
 OSERDES2_adr_obuf (
    .OQ(adr_w[i]),               // 1-bit Data output to pad or IODELAY2
    .SHIFTOUT1(), // 1-bit Cascade data output
    .SHIFTOUT2(), // 1-bit Cascade 3-state output
    .SHIFTOUT3(), // 1-bit Cascade differential data output
    .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
    .TQ(tria_w[i]),               // 1-bit 3-state output to pad or IODELAY2
    .CLK0(hclk_i),           // 1-bit I/O clock input
    .CLK1(),           // 1-bit Secondary I/O clock input
    .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
    // D1 - D4: 1-bit (each) Parallel data inputs
    .D1(adr_i[i]),
    .D2(adr_i[i]),
    .D3(adr_i[i]),
    .D4(adr_i[i]),
    .IOCE(stb_i),           // 1-bit Data strobe input
    .OCE(1'b1),             // 1-bit Clock enable input
    .RST(rst_i),             // 1-bit Asynchrnous reset input
    .SHIFTIN1(),   // 1-bit Cascade data input
    .SHIFTIN2(),   // 1-bit Cascade 3-state input
    .SHIFTIN3(),   // 1-bit Cascade differential data input
    .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
    // T1 - T4: 1-bit (each) 3-state control inputs
    .T1(1'b0),
    .T2(1'b0),
    .T3(1'b0),
    .T4(1'b0),
    .TCE(1'b1),             // 1-bit 3-state clock enable input
    .TRAIN(1'b0)          // 1-bit Training pattern enable input
 ); 
 
 IODELAY2 #(
  .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
  .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
  .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
  .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
  .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
  .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                     // or "DIFF_PHASE_DETECTOR" 
  .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
  .ODELAY_VALUE(0),                  // Amount of taps fixed output delay (0-255)
  .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
  .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
)
IODELAY2_adr_dly (
  .BUSY(),         // 1-bit Busy output after CAL
  .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
  .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
  .DOUT(adr_d_w[i]),         // 1-bit Delayed data output
  .TOUT(TOUT),         // 1-bit Delayed 3-state output
  .CAL(1'b0),           // 1-bit Initiate calibration input
  .CE(1'b0),             // 1-bit Enable INC input
  .CLK(gclk_i),           // 1-bit Clock input
  .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
  .INC(1'b0),           // 1-bit Increment / decrement input
  .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
  .IOCLK1(),     // 1-bit Input from the I/O clock network
  .ODATAIN(adr_w[i]),   // 1-bit Output data input from output register or OSERDES2.
  .RST(rst_i),           // 1-bit Reset to zero or 1/2 of total delay period
  .T(tria_w[i])                // 1-bit 3-state input signal
);			  

OBUF #(
   .DRIVE(12),   // Specify the output drive strength
   .IOSTANDARD("SSTL15_II"), // Specify the output I/O standard
   .SLEW("FAST") // Specify the output slew rate
) OBUF_cmd_out (
   .O(adr_o[i]),     // Buffer output (connect directly to top-level port)
   .I(adr_d_w[i])      // Buffer input 
);
    end
endgenerate  

//----------------------------End of address forward-------------------------------------------
//
//------------------------DQS forward------------------------------------------------------------
generate
for (i=0; i<2; i=i+1)
  begin: dqs_out
    OSERDES2 #(
    .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
    .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
    .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
    .DATA_WIDTH(4),               // Parallel data width (2-8)
    .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
    .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
    .TRAIN_PATTERN(0)             // Training Pattern (0-15)
    )
    OSERDES2_dqs_p_out (
    .OQ(dqs_pout_w[i]),               // 1-bit Data output to pad or IODELAY2
    .SHIFTOUT1(), // 1-bit Cascade data output
    .SHIFTOUT2(), // 1-bit Cascade 3-state output
    .SHIFTOUT3(), // 1-bit Cascade differential data output
    .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
    .TQ(dqs_tpout_w[i]),               // 1-bit 3-state output to pad or IODELAY2
    .CLK0(hclk_i),           // 1-bit I/O clock input
    .CLK1(),           // 1-bit Secondary I/O clock input
    .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
    // D1 - D4: 1-bit (each) Parallel data inputs
    .D1(1'b0),
    .D2(1'b1),
    .D3(1'b0),
    .D4(1'b1),
    .IOCE(stb_i),           // 1-bit Data strobe input
    .OCE(1'b1),             // 1-bit Clock enable input
    .RST(1'b0),             // 1-bit Asynchrnous reset input
    .SHIFTIN1(),   // 1-bit Cascade data input
    .SHIFTIN2(),   // 1-bit Cascade 3-state input
    .SHIFTIN3(),   // 1-bit Cascade differential data input
    .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
    // T1 - T4: 1-bit (each) 3-state control inputs
    .T1(tri_i),
    .T2(tri_i),
    .T3(tri_i),
    .T4(tri_i),
    .TCE(tri_i),             // 1-bit 3-state clock enable input
    .TRAIN(1'b0)          // 1-bit Training pattern enable input
    );   	 

    IODELAY2 #(
    .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
    .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
    .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
    .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
    .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
    .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                 // or "DIFF_PHASE_DETECTOR" 
    .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
    .ODELAY_VALUE(0),                  // Amount of taps fixed output delay (0-255)
    .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
    .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
    )
    IODELAY2_dqs_p_dly (
    .BUSY(),         // 1-bit Busy output after CAL
    .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
    .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
    .DOUT(dqs_pdly_w[i]),         // 1-bit Delayed data output
    .TOUT(dqs_tpdly_w[i]),         // 1-bit Delayed 3-state output
    .CAL(1'b0),           // 1-bit Initiate calibration input
    .CE(1'b0),             // 1-bit Enable INC input
    .CLK(gclk_i),           // 1-bit Clock input
    .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
    .INC(1'b0),           // 1-bit Increment / decrement input
    .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
    .IOCLK1(),     // 1-bit Input from the I/O clock network
    .ODATAIN(dqs_pout_w[i]),   // 1-bit Output data input from output register or OSERDES2.
    .RST(1'b0),           // 1-bit Reset to zero or 1/2 of total delay period
    .T(dqs_tpout_w[i])                // 1-bit 3-state input signal
    );
    
      IOBUF #(
       .DRIVE(12), // Specify the output drive strength
       .IOSTANDARD("SSTL15_II"), // Specify the I/O standard
       .SLEW("FAST") // Specify the output slew rate
    ) IOBUF_dqs_p_iobuf (
       .O(O),     // Buffer output
       .IO(dqs_p_o[i]),   // Buffer inout port (connect directly to top-level port)
       .I(dqs_pdly_w[i]),     // Buffer input
       .T(dqs_tpdly_w[i])      // 3-state enable input, high=input, low=output
    );  
    
  end
endgenerate

generate
for (i=0; i<2; i=i+1)
  begin: dqs_n_out
    OSERDES2 #(
    .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
    .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
    .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
    .DATA_WIDTH(4),               // Parallel data width (2-8)
    .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
    .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
    .TRAIN_PATTERN(0)             // Training Pattern (0-15)
    )
    OSERDES2_dqs_n_out (
    .OQ(dqs_nout_w[i]),               // 1-bit Data output to pad or IODELAY2
    .SHIFTOUT1(), // 1-bit Cascade data output
    .SHIFTOUT2(), // 1-bit Cascade 3-state output
    .SHIFTOUT3(), // 1-bit Cascade differential data output
    .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
    .TQ(dqs_tnout_w[i]),               // 1-bit 3-state output to pad or IODELAY2
    .CLK0(hclk_i),           // 1-bit I/O clock input
    .CLK1(),           // 1-bit Secondary I/O clock input
    .CLKDIV(gclk_i),       // 1-bit Logic domain clock input
    // D1 - D4: 1-bit (each) Parallel data inputs
    .D1(1'b1),
    .D2(1'b0),
    .D3(1'b1),
    .D4(1'b0),
    .IOCE(stb_i),           // 1-bit Data strobe input
    .OCE(1'b1),             // 1-bit Clock enable input
    .RST(1'b0),             // 1-bit Asynchrnous reset input
    .SHIFTIN1(),   // 1-bit Cascade data input
    .SHIFTIN2(),   // 1-bit Cascade 3-state input
    .SHIFTIN3(),   // 1-bit Cascade differential data input
    .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
    // T1 - T4: 1-bit (each) 3-state control inputs
    .T1(tri_i),
    .T2(tri_i),
    .T3(tri_i),
    .T4(tri_i),
    .TCE(1'b1),             // 1-bit 3-state clock enable input
    .TRAIN(1'b0)          // 1-bit Training pattern enable input
    );   	 

    IODELAY2 #(
    .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
    .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
    .DELAY_SRC("ODATAIN"),                  // "IO", "ODATAIN" or "IDATAIN" 
    .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
    .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
    .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                 // or "DIFF_PHASE_DETECTOR" 
    .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
    .ODELAY_VALUE(0),                  // Amount of taps fixed output delay (0-255)
    .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
    .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
    )
    IODELAY2_dqs_n_dly (
    .BUSY(),         // 1-bit Busy output after CAL
    .DATAOUT(DATAOUT),   // 1-bit Delayed data output to ISERDES/input register
    .DATAOUT2(DATAOUT2), // 1-bit Delayed data output to general FPGA fabric
    .DOUT(dqs_ndly_w[i]),         // 1-bit Delayed data output
    .TOUT(dqs_tndly_w[i]),         // 1-bit Delayed 3-state output
    .CAL(1'b0),           // 1-bit Initiate calibration input
    .CE(1'b0),             // 1-bit Enable INC input
    .CLK(gclk_i),           // 1-bit Clock input
    .IDATAIN(IDATAIN),   // 1-bit Data input (connect to top-level port or I/O buffer)
    .INC(1'b0),           // 1-bit Increment / decrement input
    .IOCLK0(hclk_i),     // 1-bit Input from the I/O clock network
    .IOCLK1(),     // 1-bit Input from the I/O clock network
    .ODATAIN(dqs_nout_w[i]),   // 1-bit Output data input from output register or OSERDES2.
    .RST(1'b0),           // 1-bit Reset to zero or 1/2 of total delay period
    .T(dqs_tnout_w[i])                // 1-bit 3-state input signal
    );

    IOBUF #(
     .DRIVE(12), // Specify the output drive strength
     .IOSTANDARD("SSTL15_II"), // Specify the I/O standard
     .SLEW("FAST") // Specify the output slew rate
    ) IOBUF_dqs_n_iobuf (
     .O(O),     // Buffer output
     .IO(dqs_n_o[i]),   // Buffer inout port (connect directly to top-level port)
     .I(dqs_ndly_w[i]),     // Buffer input
     .T(dqs_tndly_w[i])      // 3-state enable input, high=input, low=output
    );  


  end
endgenerate
//----------------------End of DQS forward------------------------------------------------------


endmodule