//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Module to instantiate IDDR2 privitives for the DDR3 memory
module ddr_in_buf (
  clk_i, rst_i, dq_i,
  dq_p_o, dq_n_o
  );
  
  input clk_i;                    // Input clock signal
  input rst_i;                    // Input reset signal
  input [15:0] dq_i;              // Data input from DDR3
  output [15:0] dq_p_o;           // Data captured on the posedge of clock
  output [15:0] dq_n_o;           // Data captured on the negedge of clock

  genvar j;
  generate
    for (j=0; j < 16; j=j+1)
      begin: dq_in
        IDDR2 #(
           .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
           .INIT_Q0(1'b0), // Sets initial state of the Q0 output to 1'b0 or 1'b1
           .INIT_Q1(1'b0), // Sets initial state of the Q1 output to 1'b0 or 1'b1
           .SRTYPE("SYNC") // Specifies "SYNC" or "ASYNC" set/reset
         ) IDDR2_dq_in (
           .Q0(dq_p_o[j]), // 1-bit output captured with C0 clock
           .Q1(dq_n_o[j]), // 1-bit output captured with C1 clock
           .C0(clk_i), // 1-bit clock input
           .C1(~clk_i), // 1-bit clock input
           .CE(1'b1), // 1-bit clock enable input
           .D(dq_i[j]),   // 1-bit DDR data input
           .R(1'b0),   // 1-bit reset input
           .S(1'b0)    // 1-bit set input
               );			   
      end
  endgenerate
  // End of IDDR2_inst instantiation
					
					
  
endmodule
