//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
module ddr_out_buf (
  clk_p_i, clk_n_i, buf_clk_i, rst_i,dq_p_i, dq_n_i, clk_en_i, ldmp_i, ldmn_i,
  udmp_i, udmn_i,  
  dq_out_o, ldm_o, udm_o, ldqs_p_o, ldqs_n_o, udqs_p_o, udqs_n_o, ddr_data_o,
  dqs_drv_o
  );
  
  input clk_p_i;                   // Buffered input positive clock
  input clk_n_i;                   // Buffered input negative clock
  input buf_clk_i;                 // Buffered clock input (for DQS lines)
  input rst_i;                     // Input reset signal
  input [15:0] dq_p_i;             // Byte to be shifted on the posedge
  input [15:0] dq_n_i;             // Byte to be shifted on the negedge
  input clk_en_i;                  // Clock enable input		 
  input ldmp_i;                    // Input for Lower byte data mask on the posedge
  input ldmn_i;                    // Input for Lower byte data mask on the negedge
  input udmp_i;                    // Input for Upper byte data mask on the posedge
  input udmn_i;                    // Input for Upper byte data mask on the negedge
  output [15:0] dq_out_o ;         // Output data					
  output ldm_o;                    // Output Lower byte data mask
  output udm_o;                    // Output Upper byte data mask
  output ldqs_p_o;                 // Data clock for DDR3
  output ldqs_n_o;                 //
  output udqs_p_o;                 //
  output udqs_n_o;                 //										
  output [15:0] ddr_data_o;        // Data from DDR3
  output [3:0] dqs_drv_o;          // DQS drive for DDR read
  
  // Internal registers and wires
  wire [15:0] dq_out_w;														   
  wire [15:0] data_en_w;
  wire [3:0] dqs_w;
  wire [3:0] data_en_dqs_w;
  wire [3:0] dqs_out_w;		
  
  assign ldqs_p_o = dqs_out_w[0];
  assign ldqs_n_o = dqs_out_w[1];
  assign udqs_p_o = dqs_out_w[2];
  assign udqs_n_o = dqs_out_w[3];
  
//      ODDR2     : In order to incorporate this function into the design,
//     Verilog   : the following instance declaration needs to be placed
//    instance   : in the body of the design code.  The instance name
//   declaration : (ODDR2_inst) and/or the port declarations within the
//      code     : parenthesis may be changed to properly reference and
//               : connect this function to the design.  Delete or comment
//               : out inputs/outs that are not necessary.

//  <-----Cut code below this line---->

   // ODDR2: Output Double Data Rate Output Register with Set, Reset
   //        and Clock Enable.
   //        Spartan-6
   // Xilinx HDL Language Template, version 12.3

  genvar j;
   generate
      for (j=0; j < 16; j=j+1) 
        begin: lab1
          ODDR2 #(
            .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
            .INIT(1'b0),    // Sets initial state of the Q output to 1'b0 or 1'b1
       //     .SRTYPE("ASYNC") // Specifies "SYNC" or "ASYNC" set/reset
         ) bit (
            .Q(dq_out_w[j]),   // 1-bit DDR output data
            .C0(clk_p_i),      // 1-bit clock input
            .C1(clk_n_i),      // 1-bit clock input
            .CE(clk_en_i),     // 1-bit clock enable input
            .D0(dq_p_i[j]),    // 1-bit data input (associated with C0)
            .D1(dq_n_i[j]),    // 1-bit data input (associated with C1)
            .R(~rst_i)         // 1-bit reset input
       //     .S(1'b0)         // 1-bit set input   
               );        
        end
   endgenerate
 
   
  ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0),    // Sets initial state of the Q output to 1'b0 or 1'b1
  //      .SRTYPE("ASYNC")
   ) ldm (
      .Q(ldm_o),   // 1-bit DDR output data
      .C0(clk_p_i),       // 1-bit clock input
      .C1(clk_n_i),       // 1-bit clock input
      .CE(clk_en_i),      // 1-bit clock enable input
      .D0(ldm_p_i),    // 1-bit data input (associated with C0)
      .D1(ldm_n_i)     // 1-bit data input (associated with C1)
   );			   
   ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0),    // Sets initial state of the Q output to 1'b0 or 1'b1
//      .SRTYPE("ASYNC")
   ) udm (
      .Q(udm_o),   // 1-bit DDR output data
      .C0(clk_p_i),       // 1-bit clock input
      .C1(clk_n_i),       // 1-bit clock input
      .CE(clk_en_i),      // 1-bit clock enable input
      .D0(udm_p_i),    // 1-bit data input (associated with C0)
      .D1(udm_p_o)     // 1-bit data input (associated with C1)
   );
 //------------------------Tristate buffers----------------------------------  
   generate
     for (j=0; j < 16; j=j+1)
       begin: lab2
        ODDR2 #(
          .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
          .INIT(1'b0),    // Sets initial state of the Q output to 1'b0 or 1'b1
    //      .SRTYPE("ASYNC")
       ) data_en (
          .Q(data_en_w[j]),   // 1-bit DDR output data
          .C0(clk_p_i),       // 1-bit clock input
          .C1(clk_n_i),       // 1-bit clock input
          .CE(1'b1),      // 1-bit clock enable input
          .D0(~clk_en_i),    // 1-bit data input (associated with C0)
          .D1(~clk_en_i)     // 1-bit data input (associated with C1)
                  );																       
       end     
   endgenerate
  	 
  generate
    for (j=0; j < 16; j=j+1)
      begin: bidir_buf
        IOBUF #(
        .DRIVE(12), // Specify the output drive strength
        .IOSTANDARD("DEFAULT"), // Specify the I/O standard
        .SLEW("FAST") // Specify the output slew rate
       ) IOBUF_bidir (
        .O(ddr_data_o[j]),     // Buffer output
        .IO(dq_out_o[j]),   // Buffer inout port (connect directly to top-level port)
        .I(dq_out_w[j]),     // Buffer input
        .T(data_en_w[j])      // 3-state enable input, high=input, low=output
                );
      end
   endgenerate

//----------------------End of tristate buffers-----------------------------   

//------------------------DQS clocks----------------------------------------
  ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0)    // Sets initial state of the Q output to 1'b0 or 1'b1
//      .SRTYPE("SYNC") // Specifies "SYNC" or "ASYNC" set/reset
   ) clock_ldqs_p (
      .Q(dqs_w[0]),   // 1-bit DDR output data
      .C0(buf_clk_i),   // 1-bit clock input
      .C1(~buf_clk_i),   // 1-bit clock input
      .CE(clk_en_i), // 1-bit clock enable input
      .D0(1'b1), // 1-bit data input (associated with C0)
      .D1(1'b0) // 1-bit data input (associated with C1)
 //     .R(1'b0),   // 1-bit reset input
 //     .S(1'b0)    // 1-bit set input
   );
   ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0)    // Sets initial state of the Q output to 1'b0 or 1'b1
//      .SRTYPE("SYNC") // Specifies "SYNC" or "ASYNC" set/reset
   ) clock_ldqs_n (
      .Q(dqs_w[1]),   // 1-bit DDR output data
      .C0(buf_clk_i),   // 1-bit clock input
      .C1(~buf_clk_i),   // 1-bit clock input
      .CE(clk_en_i), // 1-bit clock enable input
      .D0(1'b0), // 1-bit data input (associated with C0)
      .D1(1'b1) // 1-bit data input (associated with C1)
 //     .R(1'b0),   // 1-bit reset input
 //     .S(1'b0)    // 1-bit set input
   );
   ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0)    // Sets initial state of the Q output to 1'b0 or 1'b1
//      .SRTYPE("SYNC") // Specifies "SYNC" or "ASYNC" set/reset
   ) clock_udqs_p (
      .Q(dqs_w[2]),   // 1-bit DDR output data
      .C0(buf_clk_i),   // 1-bit clock input
      .C1(~buf_clk_i),   // 1-bit clock input
      .CE(clk_en_i), // 1-bit clock enable input
      .D0(1'b1), // 1-bit data input (associated with C0)
      .D1(1'b0) // 1-bit data input (associated with C1)
 //     .R(1'b0),   // 1-bit reset input
 //     .S(1'b0)    // 1-bit set input
   );
   ODDR2 #(
      .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
      .INIT(1'b0)    // Sets initial state of the Q output to 1'b0 or 1'b1
//      .SRTYPE("SYNC") // Specifies "SYNC" or "ASYNC" set/reset
   ) clock_udqs_n (
      .Q(dqs_w[3]),   // 1-bit DDR output data
      .C0(buf_clk_i),   // 1-bit clock input
      .C1(~buf_clk_i),   // 1-bit clock input
      .CE(clk_en_i), // 1-bit clock enable input
      .D0(1'b0), // 1-bit data input (associated with C0)
      .D1(1'b1) // 1-bit data input (associated with C1)
 //     .R(1'b0),   // 1-bit reset input
 //     .S(1'b0)    // 1-bit set input
   );

   generate
     for (j=0; j < 4; j=j+1)
       begin: lab4
        ODDR2 #(
          .DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1" 
          .INIT(1'b0),    // Sets initial state of the Q output to 1'b0 or 1'b1
    //      .SRTYPE("ASYNC")
       ) data_en_dqs (
          .Q(data_en_dqs_w[j]),   // 1-bit DDR output data
          .C0(buf_clk_i),       // 1-bit clock input
          .C1(~buf_clk_i),       // 1-bit clock input
          .CE(1'b1),      // 1-bit clock enable input
          .D0(~clk_en_i),    // 1-bit data input (associated with C0)
          .D1(~clk_en_i)     // 1-bit data input (associated with C1)
                  );																       
       end     
   endgenerate

   
  generate
    for (j=0; j < 4; j=j+1)
      begin: dqs_bidir
        IOBUF #(
          .DRIVE(12), // Specify the output drive strength
          .IOSTANDARD("DEFAULT"), // Specify the I/O standard
          .SLEW("SLOW") // Specify the output slew rate
        ) IOBUF_dqs_bidir (
          .O(dqs_drv_o[j]),     // Buffer output
          .IO(dqs_out_w[j]),   // Buffer inout port (connect directly to top-level port)
          .I(dqs_w[j]),     // Buffer input
          .T(data_en_dqs_w[j])      // 3-state enable input, high=input, low=output
               );
      end
  endgenerate
  
   // End of IOBUF_inst instantiation
					
					

//----------------------End of DQS clocks-----------------------------------
									
endmodule