//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Top file of the SOPC
//
// Wishbone bus frequency is 33MHz
// DDR3 forwarded frequency (clk_p_o, clk_n_o) is 333 MHz
// 
	 
module sopc (
  clk_p_i, clk_n_i, clk_i, btn_i, sw_i,
  // DDR3 interface						
  odt_o, reset_n_o, cke_o, adr_o, adr_na_o, clk_p_o, clk_n_o,
  ras_n_o, cas_n_o, we_n_o, ba_o, dm_o,
  dq_io, ldqs_p_io, ldqs_n_io, udqs_p_io, udqs_n_io,
  // UART interface
  sin_i, sout_o,
  // Test I/O
  led_o 
  );	 
  input clk_p_i;            // Positive differential input clock signal
  input clk_n_i;            // Negative differential input clock signal
  input clk_i;              // Single ended clock input
  input [3:0] btn_i;        // Input buttons
  input [3:0] sw_i;
  // DDR3 interface
  output odt_o;
  output reset_n_o;
  output cke_o;
  output [12:0] adr_o;						
  output [1:0] adr_na_o;
  output clk_p_o;
  output clk_n_o;	  
  output ras_n_o;
  output cas_n_o;
  output we_n_o;
  output [2:0] ba_o;
  output [1:0] dm_o;
  inout [15:0]  dq_io;       						
  inout ldqs_p_io;
  inout ldqs_n_io;
  inout udqs_p_io;
  inout udqs_n_io;
  // UART interface
  input sin_i;
  output sout_o;
  // Test I/O
  output [3:0] led_o;       // LED's output	  
  
  reg [1:0] adr_na_o = 2'b00;            // These address lines not used but must be set to zero
  reg init_done = 1'b0;
  reg [15:0] init_counter = 16'h0;
  wire rst_w = (~init_done) ? 1'b0 : ~btn_i[0]; 
  
  wire pll_out1_w;
  wire pll_out2_w;
  wire pll_out3_w;	

  wire low_clk = pll_out3_w;	 
  wire pll_low_clk = lclk_buf_w;
  
  wire locked_w;                                                                                                                                                                                                                  	
	
  wire tri_w; 
  
  wire [31:0] wb_adr_w;
  wire [31:0] wb_data_w;
  wire wb_we_w;
  wire [3:0] wb_sel_w;
  wire wb_stb_w;
  wire wb_ack_w;
  wire ddr_cyco_w;
  wire [31:0] ddr_datai_w;
  wire ddr_acki_w; 
  
  wire uart_cyco_w;
  wire [31:0] uart_adri_w;
  wire [31:0] uart_datai_w;
  wire uart_wei_w;
  wire [3:0] uart_seli_w;
  wire uart_stbi_w;
  wire uart_cyci_w;
  wire uart_acki_w;
  
  wire [3:0] test_ddr_w;
  wire [3:0] test2_ddr_w;
  wire [3:0] test3_ddr_w;
  
  wire lclk_buf_w;
  wire stb_w;				
  wire ioclk_w;		
  
  wire [15:0] bit1_in;
  wire [15:0] bit2_in;
  wire [15:0] bit3_in;
  wire [15:0] bit4_in;
  wire [15:0] bit1_out;
  wire [15:0] bit2_out;
  wire [15:0] bit3_out;
  wire [15:0] bit4_out;
  wire [15:0] delay_in_w;
  wire [15:0] delay_out_w;		  			    		   
  wire io_clb_w;
  wire io_rst_w;
  wire [15:0] io_busy;
  wire io_busy_w = |io_busy;
 
  wire [15:0] tri_oserdes_w;
  wire [15:0] tri_delay_w;
  wire [15:0] dout_w;
  wire [15:0] idatain_w;
  
  wire ddr_clk_fbin;
  wire [7:0] mask_w;
	
  reg clk_rst = 1'b1;
  reg io_rst = 1'b1;
  
  wire ras_n_w;
  wire cas_n_w;
  wire we_n_w;
  wire cke_w;
  wire [12:0] adr_w;
  wire [2:0] ba_w;
	 											
  assign led_o = (sw_i == 4'h0) ? test_ddr_w :
                 (sw_i == 4'h1) ? test2_ddr_w :
                 (sw_i == 4'h2) ? test3_ddr_w : 4'b1010;

//------------------ChipScope logic-----------------------------
/*
  wire [35:0] ila_cntr_w;					  
  wire [35:0] vio_cntr_w;
						 

chipscope_ila_v1_05_a_0 first_ila (
    .CONTROL(ila_cntr_w), // INOUT BUS [35:0]
    .CLK(pll_out6_w), // IN
    .TRIG0(test_ddr_w), //IN BUS [3:0]
    .TRIG1({bit1_out,bit2_out,bit3_out,bit4_out}), // IN BUS [63:0]
    .TRIG2({mask_w,12'h0})      // IN BUS [19:0]
	 
);

chipscope_icon_v1_06_a_0 first_icon (
    .CONTROL0(ila_cntr_w) // INOUT BUS [35:0]
);
*/ 

//------------End of ChipScope logic----------------------------
																
//-------------------Clocks block-------------------------------																
   clk_wiz_v3_2_0 ddr_clocks
   (// Clock in ports
    .CLK_IN1_P          (clk_p_i),    // IN
    .CLK_IN1_N          (clk_n_i),    // IN
    .CLKFB_IN           (ddr_clk_fbin),     // IN
    // Clock out ports
    .CLK_OUT1           (pll_out1_w),     // OUT 667 MHz
    .CLK_OUT2           (pll_out2_w),     // OUT 167 MHz
    .CLK_OUT3           (pll_out3_w),     // OUT 83 MHz
    .CLKFB_OUT          (ddr_clk_fbin),    // OUT
    // Status and control signals
			  
    .RESET              (clk_rst),        // IN
    .LOCKED             (locked_w));      // OUT	
//--------------------End of clocks block------------------------    
//					
//-----------------Global clock buffers--------------------------
   BUFG BUFG_bufg2 (
      .O(lclk_buf_w), // 1-bit Clock buffer output
      .I(pll_out2_w)  // 1-bit Clock buffer input
   );	 

   BUFPLL #(
      .DIVIDE(4),           // DIVCLK divider (1-8)
      .ENABLE_SYNC("TRUE")  // Enable synchrnonization between PLL and GCLK (TRUE/FALSE)
   )
   BUFPLL_input (
      .IOCLK(ioclk_w),               // 1-bit Output I/O clock
      .LOCK(),                 // 1-bit Synchronized LOCK output
      .SERDESSTROBE(stb_w), // 1-bit Output SERDES strobe (connect to ISERDES2/OSERDES2)
      .GCLK(lclk_buf_w),                 // 1-bit BUFG clock input
      .LOCKED(locked_w),             // 1-bit LOCKED input from PLL
      .PLLIN(pll_out1_w)                // 1-bit Clock input from PLL
   );				
   
//----------------End of global clock buffers--------------------
genvar i;					 
//
//------------------------------DS forward------------------------------------ 
//--------------------------Output buffer logic-------------------------------
  generate
    for (i=0; i<16; i=i+1)
      begin: obuf
      OSERDES2 #(
      .BYPASS_GCLK_FF("FALSE"),     // Bypass CLKDIV syncronization registers (TRUE/FALSE)
      .DATA_RATE_OQ("SDR"),         // Output Data Rate ("SDR" or "DDR")
      .DATA_RATE_OT("SDR"),         // 3-state Data Rate ("SDR" or "DDR")
      .DATA_WIDTH(4),               // Parallel data width (2-8)
      .OUTPUT_MODE("SINGLE_ENDED"), // "SINGLE_ENDED" or "DIFFERENTIAL" 
      .SERDES_MODE("NONE"),         // "NONE", "MASTER" or "SLAVE" 
      .TRAIN_PATTERN(0)             // Training Pattern (0-15)
   )
   OSERDES2_obuf (
      .OQ(delay_out_w[i]),               // 1-bit Data output to pad or IODELAY2
      .SHIFTOUT1(), // 1-bit Cascade data output
      .SHIFTOUT2(), // 1-bit Cascade 3-state output
      .SHIFTOUT3(), // 1-bit Cascade differential data output
      .SHIFTOUT4(), // 1-bit Cascade differential 3-state output
      .TQ(tri_oserdes_w[i]),               // 1-bit 3-state output to pad or IODELAY2
      .CLK0(ioclk_w),           // 1-bit I/O clock input
      .CLK1(),           // 1-bit Secondary I/O clock input
      .CLKDIV(lclk_buf_w),       // 1-bit Logic domain clock input
      // D1 - D4: 1-bit (each) Parallel data inputs
      .D1(bit1_out[i]),
      .D2(bit2_out[i]),
      .D3(bit3_out[i]),
      .D4(bit4_out[i]),
      .IOCE(stb_w),           // 1-bit Data strobe input
      .OCE(1'b1),             // 1-bit Clock enable input
      .RST(io_rst_w),             // 1-bit Asynchrnous reset input
      .SHIFTIN1(),   // 1-bit Cascade data input
      .SHIFTIN2(),   // 1-bit Cascade 3-state input
      .SHIFTIN3(),   // 1-bit Cascade differential data input
      .SHIFTIN4(),   // 1-bit Cascade differential 3-state input
      // T1 - T4: 1-bit (each) 3-state control inputs
      .T1(tri_w),
      .T2(tri_w),
      .T3(tri_w),
      .T4(tri_w),
      .TCE(1'b1),             // 1-bit 3-state clock enable input
      .TRAIN(1'b0)          // 1-bit Training pattern enable input
   );   	 
      	 
      end
  endgenerate

//-----------------------End of output buffer logic---------------------------		

//-------------------------Input buffer logic---------------------------------	
			
  generate
    for (i=0; i<16; i=i+1)
      begin: ibuf
      	   ISERDES2 #(
      .BITSLIP_ENABLE("FALSE"),      // Enable Bitslip Functionality (TRUE/FALSE)
      .DATA_RATE("SDR"),             // Data-rate ("SDR" or "DDR")
      .DATA_WIDTH(4),                // Parallel data width selection (2-8)
      .INTERFACE_TYPE("NETWORKING"), // "NETWORKING", "NETWORKING_PIPELINED" or "RETIMED" 
      .SERDES_MODE("NONE")           // "NONE", "MASTER" or "SLAVE" 
   )
   ISERDES2_ibuf (
      .CFB0(),           // 1-bit Clock feed-through route output
      .CFB1(),           // 1-bit Clock feed-through route output
      .DFB(),             // 1-bit Feed-through clock output
      .FABRICOUT(), // 1-bit Unsynchrnonized data output
      .INCDEC(),       // 1-bit Phase detector output
      // Q1 - Q4: 1-bit (each) Registered outputs to FPGA logic
      .Q1(bit1_in[i]),
      .Q2(bit2_in[i]),
      .Q3(bit3_in[i]),
      .Q4(bit4_in[i]),
      .SHIFTOUT(),   // 1-bit Cascade output signal for master/slave I/O
      .VALID(),         // 1-bit Output status of the phase detector
      .BITSLIP(1'b0),     // 1-bit Bitslip enable input
      .CE0(1'b1),             // 1-bit Clock enable input
      .CLK0(ioclk_w),           // 1-bit I/O clock network input
      .CLK1(),           // 1-bit Secondary I/O clock network input
      .CLKDIV(lclk_buf_w),       // 1-bit FPGA logic domain clock input
      .D(delay_in_w[i]),                 // 1-bit Input data
      .IOCE(stb_w),           // 1-bit Data strobe input
      .RST(io_rst_w),             // 1-bit Asynchronous reset input
      .SHIFTIN(1'b0)      // 1-bit Cascade input signal for master/slave I/O
   );
      end
  endgenerate  
//-----------------------End of input buffer logic-----------------------------

//-----------------I/O delay logic---------------------------------------------

  generate
    for (i=0; i<16; i=i+1)
      begin: delay
     IODELAY2 #(
      .COUNTER_WRAPAROUND("WRAPAROUND"), // "STAY_AT_LIMIT" or "WRAPAROUND" 
      .DATA_RATE("SDR"),                 // "SDR" or "DDR" 
      .DELAY_SRC("IO"),                  // "IO", "ODATAIN" or "IDATAIN" 
      .IDELAY2_VALUE(0),                 // Delay value when IDELAY_MODE="PCI" (0-255)
      .IDELAY_MODE("NORMAL"),            // "NORMAL" or "PCI" 
      .IDELAY_TYPE("VARIABLE_FROM_ZERO"),           // "FIXED", "DEFAULT", "VARIABLE_FROM_ZERO", "VARIABLE_FROM_HALF_MAX" 
                                         // or "DIFF_PHASE_DETECTOR" 
      .IDELAY_VALUE(0),                  // Amount of taps for fixed input delay (0-255)
      .ODELAY_VALUE(30),                  // Amount of taps fixed output delay (0-255)
      .SERDES_MODE("NONE"),              // "NONE", "MASTER" or "SLAVE" 
      .SIM_TAPDELAY_VALUE(75)            // Per tap delay used for simulation in ps
   )
   IODELAY2_delay (
      .BUSY(io_busy[i]),         // 1-bit Busy output after CAL
      .DATAOUT(delay_in_w[i]),   // 1-bit Delayed data output to ISERDES/input register
      .DATAOUT2(), // 1-bit Delayed data output to general FPGA fabric
      .DOUT(dout_w[i]),         // 1-bit Delayed data output
      .TOUT(tri_delay_w[i]),         // 1-bit Delayed 3-state output
      .CAL(io_clb_w),           // 1-bit Initiate calibration input
      .CE(1'b0),             // 1-bit Enable INC input
      .CLK(lclk_buf_w),           // 1-bit Clock input
      .IDATAIN(idatain_w[i]),   // 1-bit Data input (connect to top-level port or I/O buffer)
      .INC(1'b0),           // 1-bit Increment / decrement input
      .IOCLK0(ioclk_w),     // 1-bit Input from the I/O clock network
      .IOCLK1(),     // 1-bit Input from the I/O clock network
      .ODATAIN(delay_out_w[i]),   // 1-bit Output data input from output register or OSERDES2.
      .RST(io_rst_w),           // 1-bit Reset to zero or 1/2 of total delay period
      .T(tri_oserdes_w[i])                // 1-bit 3-state input signal
   );			  
     end
   endgenerate
//-------------------End of I/O delay logic-------------------------------------
//
//---------------------Output buffers------------------------------------------- 
  generate 
    for (i=0; i<16; i=i+1)
      begin: iobuf
      IOBUF #(
      .DRIVE(12), // Specify the output drive strength
      .IOSTANDARD("SSTL15_II"), // Specify the I/O standard
      .SLEW("FAST") // Specify the output slew rate
   ) IOBUF_iobuf (
      .O(idatain_w[i]),     // Buffer output
      .IO(dq_io[i]),   // Buffer inout port (connect directly to top-level port)
      .I(dout_w[i]),     // Buffer input
      .T(tri_delay_w[i])      // 3-state enable input, high=input, low=output
   );
     
      end
  endgenerate
//------------------------End of output buffers--------------------------------   
//
//------------------------End of DQ forward------------------------------------
//
// Modules instantiation

clk_mgmt clk_mgmt_sopc (
  .hclk_i                    (ioclk_w), 
  .gclk_i                    (lclk_buf_w), 
  .rst_i                     (io_rst),
  .stb_i                     (stb_w),	
  .dm_i                      (mask_w),
  .cmd_i                     ({ras_n_w,cas_n_w,we_n_w}),
  .adr_i                     ({adr_w,ba_w}),
  .tri_i                     (tri_w),
  .clk_p_o                   (clk_p_o), 
  .clk_n_o                   (clk_n_o),
  .dm_o                      (dm_o),
  .cmd_o                     ({ras_n_o,cas_n_o,we_n_o}),
  .adr_o                     ({adr_o,ba_o}),
  .dqs_p_o                   ({ldqs_p_io,udqs_p_io}),
  .dqs_n_o                   ({ldqs_n_io,udqs_n_io})
);
   
   
ddr3_top ddr3_sopc (           
  .clk_i                      (clk_i),
  .rst_i                      (rst_w),    				  
  .clk_div_i                  (low_clk),  
  .pll_low_clk_i              (pll_low_clk),
  .dq_i                       ({bit4_in,bit3_in,bit2_in,bit1_in}),
  .io_busy_i                  (io_busy_w),
  .odt_o                      (odt_o), 
  .reset_n_o                  (reset_n_o), 
  .cke_o                      (cke_o), 
  .adr_o                      (adr_w), 
  .ras_n_o                    (ras_n_w), 
  .cas_n_o                    (cas_n_w), 
  .we_n_o                     (we_n_w), 
  .ba_o                       (ba_w), 
  .dq_o                       ({bit4_out,bit3_out,bit2_out,bit1_out}),
  .mask_o                     (mask_w),
  .dq_drv_o                   (tri_w), 			
  .io_clb_o                   (io_clb_w),
  .io_rst_o                   (io_rst_w),
  .wb_adr_i                   (wb_adr_w), 
  .wb_data_i                  (wb_data_w), 
  .wb_we_i                    (wb_we_w), 
  .wb_sel_i                   (wb_sel_w), 
  .wb_stb_i                   (wb_stb_w), 
  .wb_cyc_i                   (ddr_cyco_w), 
  .wb_data_o                  (ddr_datai_w), 
  .wb_ack_o                   (ddr_acki_w),
  .test1_o                    (test_ddr_w),
  .test2_o                    (test2_ddr_w),
  .test3_o                    (test3_ddr_w)
);

wb_host wb_host_sopc (
  .clk_i                      (clk_i),
  .rst_i                      (rst_w),
  // Signals common to all devices
  .wb_adr_o                   (wb_adr_w),
  .wb_data_o                  (wb_data_w),
  .wb_we_o                    (wb_we_w),
  .wb_sel_o                   (wb_sel_w),
  .wb_stb_o                   (wb_stb_w),
  .wb_ack_o                   (wb_ack_w),
  // Interface signals with CPU
  .cpu_adr_i                  (32'h0),
  .cpu_data_i                 (32'h0), 
  .cpu_we_i                   (1'b0), 
  .cpu_sel_i                  (4'h0), 
  .cpu_stb_i                  (1'b0), 
  .cpu_cyc_i                  (1'b0), 
  .cpu_ack_i                  (1'b0), 
  // Interface signals with DDR3 SDRAM
  .ddr_adr_i                  (32'h0), 
  .ddr_data_i                 (ddr_datai_w), 
  .ddr_we_i                   (1'b0), 
  .ddr_sel_i                  (4'h0), 
  .ddr_stb_i                  (1'b0), 
  .ddr_cyc_i                  (1'b0), 
  .ddr_ack_i                  (ddr_acki_w),
  .ddr_cyc_o                  (ddr_cyco_w), 
  // Interface signals with UART
  .uart_adr_i                 (uart_adri_w), 
  .uart_data_i                (uart_datai_w), 
  .uart_we_i                  (uart_wei_w), 
  .uart_sel_i                 (uart_seli_w), 
  .uart_stb_i                 (uart_stbi_w), 
  .uart_cyc_i                 (uart_cyci_w), 
  .uart_ack_i                 (uart_acki_w),
  .uart_cyc_o                 (uart_cyco_w), 
 // Interface signals with Ethernet controller
  .eth_adr_i                  (32'h0), 
  .eth_data_i                 (32'h0), 
  .eth_we_i                   (1'b0), 
  .eth_sel_i                  (4'h0), 
  .eth_stb_i                  (1'b0), 
  .eth_cyc_i                  (1'b0), 
  .eth_ack_i                  (1'b0),
  .eth_cyc_o                  (),
  // Interface signals with CF card controller
  .cf_adr_i                   (32'h0), 
  .cf_data_i                  (32'h0), 
  .cf_we_i                    (1'b0), 
  .cf_sel_i                   (4'h0), 
  .cf_stb_i                   (1'b0), 
  .cf_cyc_i                   (1'b0), 
  .cf_ack_i                   (1'b0),
  .cf_cyc_o                   ()
);  

uart_wb uart_sopc (
  .clk_i                      (clk_i), 
  .rst_i                      (rst_w), 
  .sin_i                      (sin_i),
  .sout_o                     (sout_o), 
  .uart_led_o                 (),
  .wb_data_i                  (wb_data_w), 
  .wb_adr_i                   (wb_adr_w), 
  .wb_we_i                    (wb_we_w), 
  .wb_sel_i                   (wb_sel_w), 
  .wb_stb_i                   (wb_stb_w), 
  .wb_ack_i                   (wb_ack_w), 
  .wb_cyc_i                   (uart_cyco_w),
  .wb_data_o                  (uart_datai_w), 
  .wb_adr_o                   (uart_adri_w), 
  .wb_we_o                    (uart_wei_w), 
  .wb_sel_o                   (uart_seli_w), 
  .wb_stb_o                   (uart_stbi_w), 
  .wb_ack_o                   (uart_acki_w), 
  .wb_cyc_o                   (uart_cyci_w)                                                   
);		

  always @ (posedge clk_i)
    begin
      if (init_counter == 16'h00FF)
        io_rst <= 1'b0;
      else if (init_counter == 16'h08FF)
        clk_rst <= 1'b0;
        
      if (init_counter != 16'h0FFF)
        begin
          init_counter <= init_counter + 1'b1;
          init_done <= 1'b0;
        end
      else
        init_done <= 1'b1;
    end


  
endmodule