//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Top module of UART system which contains UART and WB controller
//
// UART works at WB bus frequency
//
module uart_wb (
 clk_i, rst_i, sin_i,
 sout_o, uart_led_o,
 wb_data_i, wb_adr_i, wb_we_i, wb_sel_i, wb_stb_i, wb_ack_i, wb_cyc_i,
 wb_data_o, wb_adr_o, wb_we_o, wb_sel_o, wb_stb_o, wb_ack_o, wb_cyc_o  
  );
  input clk_i;                      // Input clock
  input rst_i;                      // Input reset signal
  input sin_i;                      // Serial input for the UART
  output sout_o;                    // Serial output from the UART
  output [3:0] uart_led_o;          // Test output
  input [31:0] wb_data_i;
  input [31:0] wb_adr_i;
  input wb_we_i;
  input [3:0] wb_sel_i;
  input wb_stb_i;
  input wb_ack_i;
  input wb_cyc_i;
  output [31:0] wb_data_o;
  output [31:0] wb_adr_o;
  output wb_we_o;
  output [3:0] wb_sel_o;
  output wb_stb_o;
  output wb_ack_o;
  output wb_cyc_o;
  
  // Internal registers and wires
  
  wire [7:0] uart_data_w; 
  wire uart_rd_w;
  wire uart_wr_w;
  reg [2:0] uart_adr_r = 3'b000;
  reg uart_cs_r = 1'b1;
  wire [7:0] uart_datar_w;
  wire uart_txrdy_w;
  wire uart_rxrdy_w;
  
  // Modules instantiation
  
  uart_top uart_top_wb (
  	.clk_i                      (clk_i),
  	.rst_i                      (rst_i),
  	.sin_i                      (sin_i),
  	.ctsn_i                     (),
  	.sout_o                     (sout_o),
  	.rtsn_o                     (),
  	.data_i                     (uart_data_w),
  	.rd_i                       (uart_rd_w),
  	.wr_i                       (uart_wr_w),
  	.adr_i                      (uart_adr_r),
  	.cs_i                       (uart_cs_r),
  	.data_o                     (uart_datar_w),
  	.irq_o                      (),
  	.txrdy_o                    (uart_txrdy_w),
  	.rxrdy_o                    (uart_rxrdy_w)  	 	
  );
  
  uart_cntr uart_cntr_wb (
  	.clk_i                      (clk_i), 
  	.rst_i                      (rst_i), 
  	.data_i                     (), 
  	.uart_data_i                (uart_datar_w), 		
  	.uart_txrdy_i               (uart_txrdy_w),
  	.uart_rxrdy_i               (uart_rxrdy_w),
  	.uart_we_o                  (uart_wr_w),
  	.uart_rd_o                  (uart_rd_w),
  	.data_o                     (uart_data_w),
  	.wb_data_i                  (wb_data_i),
  	.wb_adr_i                   (wb_adr_i),
  	.wb_we_i                    (wb_we_i),
  	.wb_sel_i                   (wb_sel_i),
  	.wb_stb_i                   (wb_stb_i),
  	.wb_ack_i                   (wb_ack_i),
  	.wb_cyc_i                   (wb_cyc_i),
  	.wb_data_o                  (wb_data_o),
  	.wb_adr_o                   (wb_adr_o),
  	.wb_we_o                    (wb_we_o),
  	.wb_sel_o                   (wb_sel_o),
  	.wb_stb_o                   (wb_stb_o),
  	.wb_ack_o                   (wb_ack_o),
  	.wb_cyc_o                   (wb_cyc_o),
  	.uart_led_o                 (uart_led_o) 
  );
  
endmodule
